#include "server/config.h"

#include <cjson/cJSON.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char* read_file(const char* filename)
{
    FILE* file = fopen(filename, "rb");
    if (file == NULL) {
        printf("Error opening file\n");
        return NULL;
    }

    fseek(file, 0, SEEK_END);
    long file_size = ftell(file);
    fseek(file, 0, SEEK_SET);

    char* buffer = (char*)malloc(file_size + 1);
    if (buffer == NULL) {
        printf("Memory allocation failed\n");
        fclose(file);
        return NULL;
    }

    fread(buffer, 1, file_size, file);
    buffer[file_size] = '\0';

    fclose(file);
    return buffer;
}

void config_ctor(config_t* this)
{
    this->server_port = 0;
    this->secret      = NULL;
}

void config_dtor(config_t* this)
{
    if (this->secret) free(this->secret);
}

bool config_read(config_t* this, const char* filename)
{
    char* json_data = read_file(filename);
    if (json_data == NULL) {
        return false;
    }

    cJSON* root = cJSON_Parse(json_data);
    if (root == NULL) {
        free(json_data);
        return false;
    }

    cJSON* server_port = cJSON_GetObjectItem(root, "server_port");
    if (cJSON_IsNumber(server_port)) {
        this->server_port = server_port->valueint;
    } else {
        this->server_port = 80;
    }

    cJSON* secret = cJSON_GetObjectItem(root, "secret");
    if (cJSON_IsString(secret) && secret->valuestring != NULL) {
        this->secret = strdup(secret->valuestring);
    } else {
        this->secret = strdup("me@keqing.moe");
    }

    cJSON_Delete(root);
    free(json_data);

    return true;
}
